/*
	gfx.h

	Simple graphics stuff such as setting the video mode, etc.

	Copyright (C) 2001 Sparkle Game Company
*/

#ifndef GFX_H
#define GFX_H

#include "sdtypes.h"


/* display control register */
#define DISPLAY_CTRL	0x04000000

/*	Video modes:
	
	mode				description
	---------			----------  
	GFX_MODE0			256 color tiles, 4 BG layers, 8x8 tiles
	GFX_MODE1			?
	GFX_MODE2			?
	GFX_MODE3			high-color bitmap mode (15-bit), BG2
	GFX_MODE4			256 color bitmap mode - allows for page flipping, BG2
	GFX_MODE5			high-color bitmap mode (15-bit), 160x128, (?)
*/
#define GFX_MODE0		0x00
#define GFX_MODE1		0x01
#define GFX_MODE2		0x02
#define GFX_MODE3		0x03
#define GFX_MODE4		0x04
#define GFX_MODE5		0x05

/* 
	Background layer control registers 


							  tile block
							  ----------
							 |
							/ \
		X				X X	| | 
	F E D C B A 9 8 7 6 5 4 3 2 1 0
	| |   |       | | |mosaic    \ \
	\ /   \_______/ | |-------	   \\  priority
	 |        |		| 256 color ?    \---------
	 |		  |		|------------
	 |		  | map block
	 |		  -----------
	 | screen size
	 -------------
*/
#define BG0_CTRL		0x04000008
#define BG1_CTRL		0x0400000A
#define BG2_CTRL		0x0400000C
#define BG3_CTRL		0x0400000E

/* Background layer scrolling registers */
#define BG0_SCX			0x04000010
#define BG0_SCY			0x04000012
#define BG1_SCX			0x04000014
#define BG1_SCY			0x04000016
#define BG2_SCX			0x04000018
#define BG2_SCY			0x0400001A
#define BG3_SCX			0x0400001C
#define BG3_SCY			0x0400001E

/* Background enabling and disabling bits */
#define BG0_ENABLE		0x100
#define BG1_ENABLE		0x200
#define BG2_ENABLE		0x400
#define BG3_ENABLE		0x800

#define BG0_DISABLE		0xeff
#define BG1_DISABLE		0xdff
#define BG2_DISABLE		0xbff
#define BG3_DISABLE		0x7ff

/* Background priority bits - 0 being the highest */
#define BG_PRIORITY0	0x00
#define BG_PRIORITY1	0x01
#define BG_PRIORITY2	0x02
#define BG_PRIORITY3	0x03

/* bit is set ? 256 color : 16 palettes of 16 colors */
#define BG_256COLOR		0x80

/* mosaic option */
#define BG_MOSAIC		0x40

/* shifts to set bg values */
#define BG_TILE_SHIFT	2
#define BG_MAP_SHIFT	8			

/* enabling and disabling backgrounds */
#define enable_bg0()	*((u32 *)DISPLAY_CTRL) |= BG0_ENABLE
#define enable_bg1()	*((u32 *)DISPLAY_CTRL) |= BG1_ENABLE
#define enable_bg2()	*((u32 *)DISPLAY_CTRL) |= BG2_ENABLE
#define enable_bg3()	*((u32 *)DISPLAY_CTRL) |= BG3_ENABLE

#define disable_bg0()	*((u32 *)DISPLAY_CTRL) &= BG0_DISABLE
#define disable_bg1()	*((u32 *)DISPLAY_CTRL) &= BG1_DISABLE
#define disable_bg2()	*((u32 *)DISPLAY_CTRL) &= BG2_DISABLE
#define disable_bg3()	*((u32 *)DISPLAY_CTRL) &= BG3_DISABLE

/* RGB(r,g,b)
	
	Used for high color modes.  Values should be 0-31.
*/
#define RGB(r,g,b)  ((r)+(g<<5)+(b<<10))

/* PALETTE

	Contains 256 short ints representing the RGB triplets in the following format:

	F E D C B A 9 8 7 6 5 4 3 2 1 0
	- B B B B B G G G G G R R R R R
*/

typedef u16 PALETTE[256];

/* the 2 palette addresses */
#define PALETTE_BG		0x05000000
#define PALETTE_SPRITE	0x05000200

/* the display */
extern u16 *screen;

void set_gfx_mode(u16 mode);
void set_bg_palette(u16 *pal);
void set_sprite_palette(u16 *pal);


#endif /* GFX_H */