/* 
    File: sound.h
    Desc: implementation of the sound system (see sound.h)

    Copyright 2001 Sparkle Game Company
*/

#include <allegro.h>
#include <string.h>
#include "sound.h"

#define MAX_SOUNDS  15          //max number of sounds loaded at once

typedef enum _SOUNDSTATE
{
    SS_PLAY = 0,                //plays once and stops, can be already done
    SS_LOOP,                    //looping until manually stopped
    SS_STOP                     //stopped until manually started
} SOUNDSTATE;

typedef struct _SOUNDGROUP
{
    union
    {
        MIDI *midi_pointer;     //pointer to allegro MIDI data
        SAMPLE *sample_pointer; //pointer to allegro SAMPLE data
    };
    char init_flag;             //is there a sound loaded? 1 = yes, 0 = no
    char midi_flag;             //is the sound a midi? 1 = midi, 0 = sample
    SOUNDSTATE state;           //is the sound playing, looping, stopped?
    char filename[64];          //the filename of the sound (for caching)
    int ref_count;              //number of references
} SOUNDGROUP;

static int sound_initialized = 0;
static SOUNDGROUP sounds[MAX_SOUNDS];

void rm_sresult_string(const SRESULT sresult, char string[])
{
    switch(sresult)
    {
    case SR_OK:
        strcpy(string, "OK");
        break;
    case SR_FILEERROR:
        strcpy(string, "File Error");
        break;
    case SR_ALREADYINITIALIZED:
        strcpy(string, "Already Initialized");
        break;
    case SR_ALREADYSTOPPED:
        strcpy(string, "Already Stopped");
        break;
    case SR_INVALIDPARAMETER:
        strcpy(string, "Invalid Parameter");
        break;
    case SR_SOUNDINVALID:
        strcpy(string, "Sound Invalid");
        break;
    case SR_OUTOFMEMORY:
        strcpy(string, "Out of Memory");
        break;
    case SR_GENERIC:
        strcpy(string, "Generic Error");
        break;
    case SR_UNKNOWN:
    default:
        strcpy(string, "Unknown Error");
        break;
    }
    return;
}

SRESULT rm_sound_init(void)
{
    int i;

    if(sound_initialized == 1)
    {
        return SR_ALREADYINITIALIZED;
    }

    if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) != 0)
    {
        sound_initialized = 0;
        return SR_GENERIC;
    }

    for(i=0; i<MAX_SOUNDS; i++)
    {
        sounds[i].init_flag = 0;
        sounds[i].state = SS_STOP;
    }

    sound_initialized = 1;
    return SR_OK;
}

void rm_sound_exit(void)
{
    sound_initialized = 0;
    rm_sound_free_all();

    remove_sound();
    return;
}

SRESULT rm_sound_play(const SOUND *sound_pointer)
{
    if(sound_pointer == NULL || *sound_pointer < 0 || *sound_pointer >
        MAX_SOUNDS - 1)
    {
        return SR_INVALIDPARAMETER;
    }

    if(sounds[*sound_pointer].init_flag == 0)
    {
        return SR_SOUNDINVALID;
    }

    if(sounds[*sound_pointer].midi_flag == 1)
    {
        if(play_midi(sounds[*sound_pointer].midi_pointer, 0) != 0)
        {
            return SR_GENERIC;
        }
    }
    else
    {
        if(play_sample(sounds[*sound_pointer].sample_pointer, 128,
            128, 1000, 0) != 0)
        {
            return SR_GENERIC;
        }
    }

    sounds[*sound_pointer].state = SS_PLAY;

    return SR_OK;
}

SRESULT rm_sound_loop(const SOUND *sound_pointer)
{
    if(sound_pointer == NULL || *sound_pointer < 0 || *sound_pointer >
        MAX_SOUNDS - 1)
    {
        return SR_INVALIDPARAMETER;
    }

    if(sounds[*sound_pointer].init_flag == 0)
    {
        return SR_SOUNDINVALID;
    }

    if(sounds[*sound_pointer].midi_flag == 1)
    {
        if(play_midi(sounds[*sound_pointer].midi_pointer, 1) != 0)
        {
            return SR_GENERIC;
        }
    }
    else
    {
        if(play_sample(sounds[*sound_pointer].sample_pointer, 128,
            128, 1000, 1) != 0)
        {
            return SR_GENERIC;
        }
    }

    sounds[*sound_pointer].state = SS_LOOP;

    return SR_OK;
}

SRESULT rm_sound_stop(const SOUND *sound_pointer)
{
    if(sound_pointer == NULL || *sound_pointer < 0 || *sound_pointer >
        MAX_SOUNDS - 1)
    {
        return SR_INVALIDPARAMETER;
    }

    if(sounds[*sound_pointer].init_flag == 0)
    {
        return SR_SOUNDINVALID;
    }

    if(sounds[*sound_pointer].state == SS_STOP)
    {
        return SR_SOUNDINVALID;
    }

    if(sounds[*sound_pointer].midi_flag == 1)
    {
        stop_midi();
    }
    else
    {
        stop_sample(sounds[*sound_pointer].sample_pointer);
    }

    sounds[*sound_pointer].state = SS_STOP;

    return SR_OK;

}

void rm_sound_stop_all(void)
{
    int i;
    
    for(i=0; i<MAX_SOUNDS; i++)
    {
        if(sounds[i].init_flag == 1 && sounds[i].state != SS_STOP)
        {
            rm_sound_stop((SOUND*)&i);
        }
    }
}

SRESULT rm_sound_load(const char filename[], SOUND *sound_pointer)
{
    int i, len;
    char midi;

    if(sound_pointer == NULL)
    {
        return SR_INVALIDPARAMETER;
    }

    for(i=0; i<MAX_SOUNDS; i++)
    {
        if(sounds[i].init_flag == 1 &&
            strcmp(sounds[i].filename, filename) == 0)
        {
            sounds[i].ref_count++;
            *sound_pointer = i;
            return SR_OK;
        }
    }

    len = strlen(filename);
    if(strcmp(&filename[len - 4], ".wav") == 0)
    {
        midi = 0;
    }
    else if(strcmp(&filename[len - 4], ".mid") == 0)
    {
        midi = 1;
    }
    else if(strcmp(&filename[len - 5], ".midi") == 0)
    {
        midi = 1;
    }
    else
    {
        return SR_INVALIDPARAMETER;
    }

    for(i=0; i<MAX_SOUNDS; i++)
    {
        if(sounds[i].init_flag == 0)
        {
            if(midi == 1)
            {
                sounds[i].midi_pointer = load_midi(filename);
                if(sounds[i].midi_pointer == NULL)
                {
                    return SR_FILEERROR;
                }
                sounds[i].midi_flag = 1;
                sounds[i].init_flag = 1;
                sounds[i].ref_count = 1;
                sounds[i].state = SS_STOP;
                strcpy(sounds[i].filename, filename);
                *sound_pointer = i;
                return SR_OK;
            }
            else
            {
                sounds[i].sample_pointer = load_sample(filename);
                if(sounds[i].sample_pointer == NULL)
                {
                    return SR_FILEERROR;
                }
                sounds[i].midi_flag = 0;
                sounds[i].init_flag = 1;
                sounds[i].ref_count = 1;
                sounds[i].state = SS_STOP;
                strcpy(sounds[i].filename, filename);
                *sound_pointer = i;
                return SR_OK;
            }
        }
    }

    return SR_OUTOFMEMORY;
}

SRESULT rm_sound_free(SOUND *sound_pointer)
{
    if(sound_pointer == NULL || *sound_pointer < 0 || *sound_pointer >
        MAX_SOUNDS - 1)
    {
        return SR_INVALIDPARAMETER;
    }

    if(sounds[*sound_pointer].init_flag == 0)
    {
        return SR_SOUNDINVALID;
    }

    sounds[*sound_pointer].ref_count--;
    if(sounds[*sound_pointer].ref_count != 0)
    {
        return SR_OK;
    }

    if(sounds[*sound_pointer].midi_flag == 1)
    {
        destroy_midi(sounds[*sound_pointer].midi_pointer);
    }
    else
    {
        destroy_sample(sounds[*sound_pointer].sample_pointer);
    }

    sounds[*sound_pointer].init_flag = 0;
    return SR_OK;
}

void rm_sound_free_all(void)
{
    int i;

    for(i=0; i<MAX_SOUNDS; i++)
    {
        if(sounds[i].init_flag == 1)
        {
            if(sounds[i].midi_flag == 1)
            {
                destroy_midi(sounds[i].midi_pointer);
            }
            else
            {
                destroy_sample(sounds[i].sample_pointer);
            }

            sounds[i].init_flag = 0;
            sounds[i].ref_count = 0;
        }
    }
}