/* 
    File: sound.h
    Desc: public interfaces for sound module, sound and music functions

    Copyright 2001 Sparkle Game Company
*/

#ifndef __SOUND_H__
#define __SOUND_H__

typedef unsigned int SOUND;
typedef enum _SRESULT
{
    SR_OK = 0,
    SR_FILEERROR,
    SR_ALREADYINITIALIZED,
    SR_ALREADYSTOPPED,
    SR_INVALIDPARAMETER,
    SR_SOUNDINVALID,
    SR_OUTOFMEMORY,
    SR_GENERIC,
    SR_UNKNOWN
} SRESULT;

/*****************************************************************************
******************************************************************************
void rm_sresult_string(const SRESULT, char[]);
Input:
    const SRESULT - a sound result to convert to a string
Output:
    char[] - a C-string to write to
Return:
    none
Description:
    This function converts an SRESULT to a string.  This will be
    most often used to get a string to dump to a logfile, or
    debug output, or to send an error message to the user.
******************************************************************************
*****************************************************************************/
void rm_sresult_string(const SRESULT, char[]);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_init(void);
Input:
    none
Output:
    none
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function initializes the sound system.  This should be called
    before any of the other sound functions.  Otherwise, those functions
    will report a not initialized error
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_init(void);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_exit(void);
Input:
    none
Output:
    none
Return:
    none
Description:
    This function releases the sound system.  All memory related to the
    sound system is freed and settings are restored to how they were
    before the system was initialized.
******************************************************************************
*****************************************************************************/
void rm_sound_exit(void);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_play(const SOUND*);
Input:
    const SOUND* - the sound to be played
Output:
    none
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function plays a sound once and then the sound will stop
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_play(const SOUND*);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_loop(const SOUND*);
Input:
    const SOUND* - the sound to be played
Output:
    none
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function plays a sound repeatedly (starting over when the end is
    reached) until rm_sound_stop or rm_sound_stop_all is called.
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_loop(const SOUND*);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_stop(const SOUND*);
Input:
    const SOUND* - the sound to be stopped
Output:
    none
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function stops a sound if it is playing or looping
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_stop(const SOUND*);

/*****************************************************************************
******************************************************************************
void rm_sound_stop_all(void);
Input:
    none
Output:
    none
Return:
    none
Description:
    This function stops all sounds that are playing or looping
******************************************************************************
*****************************************************************************/
void rm_sound_stop_all(void);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_load(const char[], SOUND*);
Input:
    const char[] - the file to load the sound from (name: wav, mid, or midi)
Output:
    SOUND* - a pointer to the loaded sound
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function loads a sound
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_load(const char[], SOUND*);

/*****************************************************************************
******************************************************************************
SRESULT rm_sound_free(const SOUND*);
Input:
    const SOUND* - the sound to be freed
Output:
    none
Return:
    SRESULT - sound result, returns SR_OK on success
Description:
    This function frees a sound
******************************************************************************
*****************************************************************************/
SRESULT rm_sound_free(SOUND*);

/*****************************************************************************
******************************************************************************
void rm_sound_free_all(void);
Input:
    none
Output:
    none
Return:
    none
Description:
    This function frees all loaded sounds
******************************************************************************
*****************************************************************************/
void rm_sound_free_all(void);

#endif//__SOUND_H__